
import matplotlib.pyplot as plt
import numpy as np
import scipy.integrate
import scipy.ndimage as ndi


from . import plume


from scipy.optimize import fmin_cobyla

P = (0.5, 2)


def find_min_distance_to_curve(p,c,x):

    def objective(C):
        x,y = C
        return np.sqrt((x - p[0])**2 + (y - p[1])**2)

    def c1(C):
        return np.polyval(C,x) - y

    X = fmin_cobyla(objective, x0=[0.5,0.5], cons=[c1])

    return X


def plume_function(x,y,x0=0.0,y0=0.0, K=4, u=10):
    """\
    x: distance from source
    u: mean wind speed along plume
    """
    Q = 1.0

    sigma = np.sqrt(2.0 * K * x / u)

    return Q / (2.0 * np.pi * sigma) * np.exp( -(y-y0)**2 / (2.0 * sigma**2))






def gauss(x,y, x0=0.0, y0=0.0):

    Q = 1.0
    u = 1.0

    sigma = 1. * (x-x0)**0.894

    A = Q / (np.sqrt(2.0 * np.pi) * sigma * u)
    B = np.exp(-0.5 * (y-y0)**2 / sigma**2)

    return A * B



def find_all_roots(hh, dd, d, h_max=5000.0):
    """\
    Find all heights (hh), below hh_max where wind direction (dd) equals
    given wind direction (d).
    """

    founds = []

    for i in range(hh.size-1):
        if dd[i] < dd[i+1]:
            dmin, dmax = dd[i], dd[i+1]
            hmin, hmax = hh[i], hh[i+1]
        else:
            dmin, dmax = dd[i+1], dd[i]
            hmin, hmax = hh[i+1], hh[i]

        if hmin > h_max:
            continue

        if dmin <= d < dmax:
            d0 = np.interp(d, [dmin, dmax], [hmin, hmax])
            founds.append(d0)

    return founds




def gauss_puff(x,y,sigma):
    pass


def arc_length_of_poly(c, a, b):

    x = np.linspace(a,b)
    s = np.sqrt( 1 + np.polyval(np.polyder(c), x)**2 )

    return scipy.integrate.simps(s,x)



def flux_inversion(lon, lat, field, clouds, is_plume, h, uwind, vwind,
        degree=3, origin=None, **kwargs):


    # location of plume
    lon_o, lat_o = plume.get_source_origin(origin)
    i_o, j_o = plume.nearest_index(lon_o, lat_o, lon, lat)

    # get location of plume and XCO2 values (plus wind etc.)
    lon0 = lon[is_plume]
    lat0 = lat[is_plume]
    values = field[is_plume]

    # polyfit center of plume (TODO: should work with general 2D curves)
    x = np.linspace(lon0.min(), lon0.max())
    w = np.ones(lon0.shape)

    # make sure center line goes through point source
    lon0 = np.append(lon0, lon_o)
    lat0 = np.append(lat0, lat_o)
    w = np.append(w, 1000.0)

    c = np.polyfit(lon0,lat0,degree, w=w)

    # estimate cardinal wind direction (and plume height) TODO
    dd = (270.0 - np.rad2deg(np.arctan2(vwind,uwind))) % 360
    d = (270 - np.rad2deg(np.arctan(m))) % 360

    # estimate wind direction (at center line) TODO
    heights = find_all_roots(hh, dd, d, h_max=5000.0)

    # make plume bigger to include background
    struct = ndi.generate_binary_structure(2, 2)
    full_area = ndi.binary_dilation(is_plume, struct, iterations=5) # TODO iterations?

    # compute for all ground pixels distance to center line
    lon1 = lon[full_plume]
    lat1 = lat[full_plume]

    distance = None # TODO


    # set up inversion

    # compute 2D Gauss plume
    # (x: distance from source, y: distance from line)

    # flux inversion (fit Q)



def test():
    x = np.linspace(0.1,10,501)
    y = np.linspace(-5,5,501)

    x,y = np.meshgrid(x,y)

    rho = plume(x,y, sigma=5.)


    plt.figure()
    plt.imshow(rho)



    return rho
